#BIBLIOTHEQUES

import csv


"""OUVERTURE ET LECTURE DES FICHIERS"""

with open("./data_arcs.csv", newline="") as csvfile:
    reader = csv.reader(csvfile, delimiter=";")
    
    adjacents = {} # Dictionnaire contenant comme clefs les sommets et
                   # comme valeurs la liste des sommets pouvant etre rejoints
                   # depuis le sommet designe par la clef.
                   # Seuls les arcs de type "favorise" sont retenus ici.
    
    for ligne in reader:
        source, interaction, cible = ligne
        if interaction == "favorise":
            if source not in adjacents: # si la clef n'existe pas encore
                adjacents[source] = [cible] # on cree l'entree
            else: # sinon on ajoute l'element a la liste des sommets adjacents
                adjacents[source].append(cible)


"""FONCTIONS"""
              
def existe_chemin_BFS(adjacents, depart, arrivee):
    a_explorer = [depart]
    deja_collectes = [depart]
    
    while len(a_explorer) != 0:
        courant = a_explorer.pop(0)
        
        if courant == arrivee:
            return(True)
        
        if courant in adjacents:
            for sommet in adjacents[courant]:
                if sommet not in deja_collectes:
                    a_explorer.append(sommet)
                    deja_collectes.append(sommet)
        
    return(False)


def calcul_chemin_BFS(adjacents, depart, arrivee):
    a_explorer = [depart]
    deja_collectes = [depart]
    chemins = {depart: [depart]}
    
    while len(a_explorer) != 0:
        courant = a_explorer.pop(0)
        
        if courant == arrivee:
            return(chemins[arrivee])
        
        if courant in adjacents:
            for sommet in adjacents[courant]:
                if sommet not in deja_collectes:
                    a_explorer.append(sommet)
                    deja_collectes.append(sommet)
                    chemins[sommet] = chemins[courant] + [sommet]
        
    return(None)

"""PROGRAMME"""

liste_ingredients = ["tomate","courgette","chou","oignon","carotte","laitue","ciboulette","thym","ail"]

dico_chemins = {}

for plante_depart in liste_ingredients :
  for plante_arrivee in liste_ingredients :
    if plante_depart != plante_arrivee :
      chemin = calcul_chemin_BFS(adjacents, plante_depart, plante_arrivee)
      dico_chemins[(plante_depart, plante_arrivee)] = chemin

#print (dico_chemins)

dico_cycles = {}

for couple in dico_chemins :
  plante_depart, plante_arrivee = couple
  aller = dico_chemins[(plante_depart, plante_arrivee)]
  retour = dico_chemins[(plante_arrivee, plante_depart)]
  if aller != None and retour != None and (plante_arrivee, plante_depart) not in dico_cycles :
    dico_cycles[(plante_depart, plante_arrivee)] = aller[:-1] + retour

#print(dico_cycles)

for cycle in dico_cycles.values() :
  compteur_ingredients = 0
  compteur_autres = 0
  deja_comptees = []
  for plante in cycle :
    if plante in liste_ingredients and plante not in deja_comptees :
      deja_comptees.append(plante)
      compteur_ingredients += 1
    elif plante not in liste_ingredients and plante not in deja_comptees :
      deja_comptees.append(plante)
      compteur_autres += 1
      
  cycle.append(compteur_ingredients)
  cycle.append(compteur_autres)

#print(dico_cycles)

max = list(dico_cycles.values())[0][-2]
for val in dico_cycles.values():
  if val[-2] > max :
    max = val[-2]

liste_cycles_max_ingredients = []
for (cle, val) in dico_cycles.items():
  if val[-2] == max :
    liste_cycles_max_ingredients.append((cle,val))

#print(liste_cycles_max_ingredients)

liste_elements_jardin = liste_cycles_max_ingredients[0][1][:-3]
#print(liste_elements_jardin)

#print(liste_ingredients)

liste_ingredients_restants = []
for plante in liste_ingredients :
  if plante not in liste_elements_jardin :
    liste_ingredients_restants.append(plante)

#print(liste_ingredients_restants)

with open("./data_sommets_categories.csv", newline="") as csvfile:
    reader = csv.reader(csvfile, delimiter=";")
    
    liste_fleurs = []
  
    for ligne in reader:
        plante, categorie = ligne
        if categorie == "fleur":
            liste_fleurs.append(plante)

dico_chemins_fleurs = {}
liste_fleurs_jardin = []

for plante_depart in liste_elements_jardin :
  for plante_arrivee in liste_fleurs :
    if plante_depart != plante_arrivee :
      chemin = calcul_chemin_BFS(adjacents, plante_depart, plante_arrivee)
      if chemin != None and len(chemin) == 2 :
        dico_chemins_fleurs[(plante_depart, plante_arrivee)] = chemin
        if plante_arrivee not in liste_fleurs_jardin :
          liste_fleurs_jardin.append(plante_arrivee)

#print(dico_chemins_fleurs)
#print(liste_fleurs_jardin)

dico_chemins_restants = {}
liste_a_parcourir = liste_fleurs_jardin + liste_elements_jardin

for plante_depart in liste_a_parcourir :
  for plante_arrivee in liste_ingredients_restants :
    if plante_depart != plante_arrivee :
      chemin = calcul_chemin_BFS(adjacents, plante_depart, plante_arrivee)
      if chemin != None and len(chemin) == 2 :
        dico_chemins_restants[(plante_depart, plante_arrivee)] = chemin

#print(dico_chemins_restants)
